#include "rtc.h"
#include "i2c_master.h"

// Convert normal decimal numbers to binary coded decimal
uint8_t bin2bcd(uint8_t val) {
  return( (val / 10 * 16) + (val % 10) );
}

// Convert binary coded decimal to normal decimal numbers
uint8_t bcd2bin(uint8_t val) {
  return( (val / 16 * 10) + (val % 16) );
}

void RTC_Now(rtc_t *rtc) {
    I2C_start(C_Ds1307WriteMode_U8);
    I2C_write(0x00);
    I2C_stop();

    I2C_start(C_Ds1307ReadMode_U8);
    rtc->sec = bcd2bin(I2C_read_ack() & 0x7F);
    rtc->min = bcd2bin(I2C_read_ack());              // read minute and return Positive ACK
    rtc->hour = bcd2bin(I2C_read_ack());     // read hour and return Negative/No ACK
    rtc->weekDay = bcd2bin(I2C_read_ack());
    rtc->date = bcd2bin(I2C_read_ack());
    rtc->month = bcd2bin(I2C_read_ack());
    rtc->year = bcd2bin(I2C_read_nack());
    I2C_stop();
}

void RTC_init(void)
{
    I2C_start(C_Ds1307WriteMode_U8);        // Connect to DS1307 by sending its ID on I2c Bus

    I2C_write(C_Ds1307ControlRegAddress_U8);// Select the Ds1307 ControlRegister to configure Ds1307
    I2C_write(0x00);                        // Write 0x00 to Control register to disable SQW-Out

    I2C_stop();                             // Stop I2C communication after initializing DS1307
}

void RTC_setDateTime(rtc_t *rtc)
{
    I2C_start(C_Ds1307WriteMode_U8);        // connect to DS1307 by sending its ID on I2c Bus

    I2C_write(C_Ds1307SecondRegAddress_U8); // Request sec RAM address at 00H

    I2C_write(bin2bcd(rtc->sec));           // Write sec from RAM address 00H
    I2C_write(bin2bcd(rtc->min));           // Write min from RAM address 01H
    I2C_write(bin2bcd(rtc->hour));          // Write hour from RAM address 02H
    I2C_write(bin2bcd(rtc->weekDay));       // Write weekDay on RAM address 03H
    I2C_write(bin2bcd(rtc->date));          // Write date on RAM address 04H
    I2C_write(bin2bcd(rtc->month));         // Write month on RAM address 05H
    I2C_write(bin2bcd(rtc->year));          // Write year on RAM address 06h

    I2C_stop();                             // Stop I2C communication after Setting the Date
}

void RTC_getDateTime(rtc_t *rtc)
{
    I2C_start(C_Ds1307WriteMode_U8);        // connect to DS1307 by sending its ID on I2c Bus

    I2C_write(C_Ds1307SecondRegAddress_U8); // Request Sec RAM address at 00H

    I2C_stop();                             // Stop I2C communication after selecting Sec Register

    I2C_start(C_Ds1307ReadMode_U8);         // connect to DS1307(Read mode) by sending its ID

    rtc->sec = bcd2bin(I2C_read_ack() & 0x7F);       // read second and return Positive ACK
    rtc->min = bcd2bin(I2C_read_ack());              // read minute and return Positive ACK
    rtc->hour = bcd2bin(I2C_read_ack() & 0x3f);      // read hour and return Negative/No ACK
    rtc->weekDay = bcd2bin(I2C_read_ack());          // read weekDay and return Positive ACK
    rtc->date = bcd2bin(I2C_read_ack());             // read Date and return Positive ACK
    rtc->month = bcd2bin(I2C_read_ack());            // read Month and return Positive ACK
    rtc->year = bcd2bin(I2C_read_nack());            // read Year and return Negative/No ACK

    I2C_stop();                             // Stop I2C communication after reading the Date
}
